@@ -50,6 +50,7 @@ static const rc_service_state_name_t rc_service_state_names[] = {
 	{ RC_SERVICE_HOTPLUGGED,  "hotplugged" },
 	{ RC_SERVICE_FAILED,      "failed" },
 	{ RC_SERVICE_SCHEDULED,   "scheduled"},
+	{ RC_SERVICE_CRASHED,     "crashed"},
 	{ 0, NULL}
 };
 
@@ -848,6 +849,10 @@ rc_service_state(const char *service)
 		}
 	}
 
+	if (state & RC_SERVICE_STARTED) {
+		if (rc_service_daemons_crashed(service))
+			state |= RC_SERVICE_CRASHED;
+	}
 	if (state & RC_SERVICE_STOPPED) {
 		dirs = ls_dir(RC_SVCDIR "/scheduled", 0);
 		TAILQ_FOREACH(dir, dirs, entries) {
