@@ -54,7 +54,6 @@ const char *usagestring = ""						\
 	"Usage: rc-status [options] <runlevel>...\n"		\
 	"   or: rc-status [options] [-a | -c | -l | -m | -r | -s | -u]";
 
-static bool test_crashed = false;
 static RC_DEPTREE *deptree;
 static RC_STRINGLIST *types;
 
@@ -76,7 +75,7 @@ print_level(const char *prefix, const char *level)
 		printf("%s\n", level);
 }
 
-static void get_uptime(const char *service, char *uptime, int uptime_size)
+static char *get_uptime(const char *service)
 {
 	RC_SERVICE state = rc_service_state(service);
 	char *start_count;
@@ -88,8 +87,8 @@ static void get_uptime(const char *service, char *uptime, int uptime_size)
 	time_t diff_hours = (time_t) 0;
 	time_t diff_mins = (time_t) 0;
 	time_t diff_secs = (time_t) 0;
+	char *uptime = NULL;
 
-	uptime[0] = '\0';
 	if (state & RC_SERVICE_STARTED) {
 		start_count = rc_service_value_get(service, "start_count");
 		start_time_string = rc_service_value_get(service, "start_time");
@@ -111,23 +110,24 @@ static void get_uptime(const char *service, char *uptime, int uptime_size)
 				diff_secs %= diff_mins * (time_t) 60;
 			}
 			if (diff_days > 0)
-				snprintf(uptime, uptime_size,
+				xasprintf(&uptime,
 						"%ld day(s) %02ld:%02ld:%02ld (%s)",
 						diff_days, diff_hours, diff_mins, diff_secs,
 						start_count);
 			else
-				snprintf(uptime, uptime_size,
+				xasprintf(&uptime,
 						"%02ld:%02ld:%02ld (%s)",
 						diff_hours, diff_mins, diff_secs, start_count);
 		}
 	}
+	return uptime;
 }
 
 static void
 print_service(const char *service)
 {
-	char status[60];
-	char uptime [40];
+	char *status = NULL;
+	char *uptime = NULL;
 	char *child_pid = NULL;
 	char *start_time = NULL;
 	int cols =  printf(" %s", service);
@@ -136,42 +136,45 @@ print_service(const char *service)
 	ECOLOR color = ECOLOR_BAD;
 
 	if (state & RC_SERVICE_STOPPING)
-		snprintf(status, sizeof(status), "stopping ");
+		xasprintf(&status, "stopping ");
 	else if (state & RC_SERVICE_STARTING) {
-		snprintf(status, sizeof(status), "starting ");
+		xasprintf(&status, "starting ");
 		color = ECOLOR_WARN;
 	} else if (state & RC_SERVICE_INACTIVE) {
-		snprintf(status, sizeof(status), "inactive ");
+		xasprintf(&status, "inactive ");
 		color = ECOLOR_WARN;
 	} else if (state & RC_SERVICE_STARTED) {
 		errno = 0;
-		if (test_crashed &&
-		    rc_service_daemons_crashed(service) &&
-		    errno != EACCES)
+		if (rc_service_daemons_crashed(service) && errno != EACCES)
 		{
 			child_pid = rc_service_value_get(service, "child_pid");
 			start_time = rc_service_value_get(service, "start_time");
 			if (start_time && child_pid)
-				snprintf(status, sizeof(status), " unsupervised ");
+				xasprintf(&status, " unsupervised ");
 			else
-				snprintf(status, sizeof(status), " crashed ");
+				xasprintf(&status, " crashed ");
 			free(child_pid);
 			free(start_time);
 		} else {
-			get_uptime(service, uptime, 40);
-			snprintf(status, sizeof(status), " started %s", uptime);
+			uptime = get_uptime(service);
+			if (uptime) {
+				xasprintf(&status, " started %s", uptime);
+				free(uptime);
+			} else
+				xasprintf(&status, " started ");
 			color = ECOLOR_GOOD;
 		}
 	} else if (state & RC_SERVICE_SCHEDULED) {
-		snprintf(status, sizeof(status), "scheduled");
+		xasprintf(&status, "scheduled");
 		color = ECOLOR_WARN;
 	} else
-		snprintf(status, sizeof(status), " stopped ");
+		xasprintf(&status, " stopped ");
 
 	errno = 0;
 	if (c && *c && isatty(fileno(stdout)))
 		printf("\n");
 	ebracket(cols, color, status);
+	free(status);
 }
 
 static void
@@ -240,8 +243,6 @@ int main(int argc, char **argv)
 	char *p, *runlevel = NULL;
 	int opt, retval = 0;
 
-	test_crashed = _rc_can_find_pids();
-
 	applet = basename_c(argv[0]);
 	while ((opt = getopt_long(argc, argv, getoptstring, longopts,
 				  (int *) 0)) != -1)
