@@ -59,13 +59,14 @@ static struct pam_conv conv = { NULL, NULL};
 #include "queue.h"
 #include "rc.h"
 #include "rc-misc.h"
+#include "rc-pipes.h"
 #include "rc-schedules.h"
 #include "_usage.h"
 #include "helpers.h"
 
 const char *applet = NULL;
 const char *extraopts = NULL;
-const char *getoptstring = "I:KN:PR:Sa:bc:d:e:g:ik:mn:op:s:tu:r:w:x:1:2:" \
+const char *getoptstring = "I:KN:PR:Sa:bc:d:e:g:ik:mn:op:s:tu:r:w:x:1:2:3:4:" \
 	getoptstring_COMMON;
 const struct option longopts[] = {
 	{ "ionice",       1, NULL, 'I'},
@@ -93,6 +94,8 @@ const struct option longopts[] = {
 	{ "exec",         1, NULL, 'x'},
 	{ "stdout",       1, NULL, '1'},
 	{ "stderr",       1, NULL, '2'},
+	{ "stdout-logger",1, NULL, '3'},
+	{ "stderr-logger",1, NULL, '4'},
 	{ "progress",     0, NULL, 'P'},
 	longopts_COMMON
 };
@@ -122,6 +125,8 @@ const char * const longopts_help[] = {
 	"Binary to start/stop",
 	"Redirect stdout to file",
 	"Redirect stderr to file",
+	"Redirect stdout to process",
+	"Redirect stderr to process",
 	"Print dots each second while waiting",
 	longopts_help_COMMON
 };
@@ -162,20 +167,20 @@ handle_signal(int sig)
 {
 	int status;
 	int serrno = errno;
-	char signame[10] = { '\0' };
+	char *signame = NULL;
 
 	switch (sig) {
 	case SIGINT:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGINT");
+		if (!signame)
+			xasprintf(&signame, "SIGINT");
 		/* FALLTHROUGH */
 	case SIGTERM:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGTERM");
+		if (!signame)
+			xasprintf(&signame, "SIGTERM");
 		/* FALLTHROUGH */
 	case SIGQUIT:
-		if (!signame[0])
-			snprintf(signame, sizeof(signame), "SIGQUIT");
+		if (!signame)
+			xasprintf(&signame, "SIGQUIT");
 		eerrorx("%s: caught %s, aborting", applet, signame);
 		/* NOTREACHED */
 
@@ -194,6 +199,9 @@ handle_signal(int sig)
 		eerror("%s: caught unknown signal %d", applet, sig);
 	}
 
+	/* free signame */
+	free(signame);
+
 	/* Restore errno */
 	errno = serrno;
 }
@@ -202,7 +210,6 @@ static char *
 expand_home(const char *home, const char *path)
 {
 	char *opath, *ppath, *p, *nh;
-	size_t len;
 	struct passwd *pw;
 
 	if (!path || *path != '~')
@@ -233,9 +240,7 @@ expand_home(const char *home, const char *path)
 		return xstrdup(home);
 	}
 
-	len = strlen(ppath) + strlen(home) + 1;
-	nh = xmalloc(len);
-	snprintf(nh, len, "%s%s", home, ppath);
+	xasprintf(&nh, "%s%s", home, ppath);
 	free(opath);
 	return nh;
 }
@@ -254,6 +259,7 @@ int main(int argc, char **argv)
 #endif
 
 	int opt;
+	size_t size = 0;
 	bool start = false;
 	bool stop = false;
 	bool oknodo = false;
@@ -275,6 +281,8 @@ int main(int argc, char **argv)
 	int tid = 0;
 	char *redirect_stderr = NULL;
 	char *redirect_stdout = NULL;
+	char *stderr_process = NULL;
+	char *stdout_process = NULL;
 	int stdin_fd;
 	int stdout_fd;
 	int stderr_fd;
@@ -287,10 +295,10 @@ int main(int argc, char **argv)
 	char *tmp, *newpath, *np;
 	char *p;
 	char *token;
-	char exec_file[PATH_MAX];
+	char *exec_file = NULL;
 	struct passwd *pw;
 	struct group *gr;
-	char line[130];
+	char *line = NULL;
 	FILE *fp;
 	size_t len;
 	mode_t numask = 022;
@@ -378,6 +386,7 @@ int main(int argc, char **argv)
 		case 'c':  /* --chuid <username>|<uid> */
 			/* DEPRECATED */
 			ewarn("WARNING: -c/--chuid is deprecated and will be removed in the future, please use -u/--user instead");
+			/* falls through */
 		case 'u':  /* --user <username>|<uid> */
 		{
 			p = optarg;
@@ -499,6 +508,14 @@ int main(int argc, char **argv)
 			redirect_stderr = optarg;
 			break;
 
+		case '3':   /* --stdout-logger "command to run for stdout logging" */
+			stdout_process = optarg;
+			break;
+
+		case '4':  /* --stderr-logger "command to run for stderr logging" */
+			stderr_process = optarg;
+			break;
+
 		case_RC_COMMON_GETOPT
 		}
 
@@ -550,6 +567,9 @@ int main(int argc, char **argv)
 		if (redirect_stdout || redirect_stderr)
 			eerrorx("%s: --stdout and --stderr are only relevant"
 			    " with --start", applet);
+		if (stdout_process || stderr_process)
+			eerrorx("%s: --stdout-logger and --stderr-logger are only relevant"
+			    " with --start", applet);
 		if (start_wait)
 			ewarn("using --wait with --stop has no effect,"
 			    " use --retry instead");
@@ -562,6 +582,15 @@ int main(int argc, char **argv)
 		if ((redirect_stdout || redirect_stderr) && !background)
 			eerrorx("%s: --stdout and --stderr are only relevant"
 			    " with --background", applet);
+		if ((stdout_process || stderr_process) && !background)
+			eerrorx("%s: --stdout-logger and --stderr-logger are only relevant"
+			    " with --background", applet);
+		if (redirect_stdout && stdout_process)
+			eerrorx("%s: do not use --stdout and --stdout-logger together",
+					applet);
+		if (redirect_stderr && stderr_process)
+			eerrorx("%s: do not use --stderr and --stderr-logger together",
+					applet);
 	}
 
 	/* Expand ~ */
@@ -577,26 +606,22 @@ int main(int argc, char **argv)
 		if (*exec == '/' || *exec == '.') {
 			/* Full or relative path */
 			if (ch_root)
-				snprintf(exec_file, sizeof(exec_file),
-				    "%s/%s", ch_root, exec);
+				xasprintf(&exec_file, "%s/%s", ch_root, exec);
 			else
-				snprintf(exec_file, sizeof(exec_file),
-				    "%s", exec);
+				xasprintf(&exec_file, "%s", exec);
 		} else {
 			/* Something in $PATH */
 			p = tmp = xstrdup(getenv("PATH"));
-			*exec_file = '\0';
+			exec_file = NULL;
 			while ((token = strsep(&p, ":"))) {
 				if (ch_root)
-					snprintf(exec_file, sizeof(exec_file),
-					    "%s/%s/%s",
-					    ch_root, token, exec);
+					xasprintf(&exec_file, "%s/%s/%s", ch_root, token, exec);
 				else
-					snprintf(exec_file, sizeof(exec_file),
-					    "%s/%s", token, exec);
-				if (exists(exec_file))
+					xasprintf(&exec_file, "%s/%s", token, exec);
+				if (exec_file && exists(exec_file))
 					break;
-				*exec_file = '\0';
+				free(exec_file);
+				exec_file = NULL;
 			}
 			free(tmp);
 		}
@@ -604,8 +629,8 @@ int main(int argc, char **argv)
 	if (start && !exists(exec_file)) {
 		eerror("%s: %s does not exist", applet,
 		    *exec_file ? exec_file : exec);
+		free(exec_file);
 		exit(EXIT_FAILURE);
-
 	}
 	if (start && retry)
 		ewarn("using --retry with --start has no effect,"
@@ -617,7 +642,10 @@ int main(int argc, char **argv)
 	if (interpreted && !pidfile) {
 		fp = fopen(exec_file, "r");
 		if (fp) {
-			p = fgets(line, sizeof(line), fp);
+			line = NULL;
+			if (getline(&line, &size, fp) == -1)
+				eerrorx("%s: %s", applet, strerror(errno));
+			p = line;
 			fclose(fp);
 			if (p != NULL && line[0] == '#' && line[1] == '!') {
 				p = line + 2;
@@ -629,7 +657,8 @@ int main(int argc, char **argv)
 				if (p[len] == '\n')
 					p[len] = '\0';
 				token = strsep(&p, " ");
-				strncpy(exec_file, token, sizeof(exec_file));
+				free(exec_file);
+				xasprintf(&exec_file, "%s", token);
 				opt = 0;
 				for (nav = argv; *nav; nav++)
 					opt++;
@@ -659,7 +688,7 @@ int main(int argc, char **argv)
 			parse_schedule(applet, NULL, sig);
 		if (pidfile) {
 			pid = get_pid(applet, pidfile);
-			if (pid == -1)
+			if (pid == -1 && errno != ENOENT)
 				exit(EXIT_FAILURE);
 		} else {
 			pid = 0;
@@ -884,6 +913,12 @@ int main(int argc, char **argv)
 				eerrorx("%s: unable to open the logfile"
 				    " for stdout `%s': %s",
 				    applet, redirect_stdout, strerror(errno));
+		}else if (stdout_process) {
+			stdout_fd = rc_pipe_command(stdout_process);
+			if (stdout_fd == -1)
+				eerrorx("%s: unable to open the logging process"
+				    " for stdout `%s': %s",
+				    applet, stdout_process, strerror(errno));
 		}
 		if (redirect_stderr) {
 			if ((stderr_fd = open(redirect_stderr,
@@ -892,13 +927,21 @@ int main(int argc, char **argv)
 				eerrorx("%s: unable to open the logfile"
 				    " for stderr `%s': %s",
 				    applet, redirect_stderr, strerror(errno));
+		}else if (stderr_process) {
+			stderr_fd = rc_pipe_command(stderr_process);
+			if (stderr_fd == -1)
+				eerrorx("%s: unable to open the logging process"
+				    " for stderr `%s': %s",
+				    applet, stderr_process, strerror(errno));
 		}
 
 		if (background)
 			dup2(stdin_fd, STDIN_FILENO);
-		if (background || redirect_stdout || rc_yesno(getenv("EINFO_QUIET")))
+		if (background || redirect_stdout || stdout_process
+				|| rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stdout_fd, STDOUT_FILENO);
-		if (background || redirect_stderr || rc_yesno(getenv("EINFO_QUIET")))
+		if (background || redirect_stderr || stderr_process
+				|| rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stderr_fd, STDERR_FILENO);
 
 		for (i = getdtablesize() - 1; i >= 3; --i)
